unit Grayform;

interface uses winprocs, wintypes, sysutils, Classes, graphics, forms, wgray;

type
  PGrayType = ^TGrayType;
  TGrayType = Record
    GrayWindow : TGrayWindow;
  end;(* TGrayRecord *)

  TGrayForm = class (TComponent)
  private
    GrayItem : PGrayType;
    GrayItems : TList;
    FOldHint : TNotifyEvent;
    FEnabled : boolean;

    procedure FreeAllItems;
    function FindGrayWindow( aForm : TForm ) : boolean;
    function FindAndRemoveGrayItem( aForm : TForm ) : boolean;
  protected
    procedure DoGrayDestroy(Sender : TObject); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure OpenGray( aForm : TForm );
    procedure CloseGray( aForm : TForm );

    property Enabled : boolean read FEnabled write FEnabled;
  end;

var
  GrayShade : TGrayForm;

implementation

procedure TGrayForm.FreeAllItems;
begin
  while GrayItems.Count > 0 do
  begin
    GrayItem := GrayItems[ 0 ];
    GrayItems.Delete( 0 );
    dispose( GrayItem );
  end;
  GrayItem := nil;
end;(* FreeAllItems *)

function TGrayForm.FindGrayWindow( aForm : TForm ) : boolean;
var ii : integer;
begin
  result := true;
  for ii := 0 to GrayItems.count-1 do
  begin
    GrayItem := GrayItems[ ii ];
    if GrayItem^.GrayWindow.GrayOwner = aForm then exit;
  end;
  result := false;
end;(* FindGrayWindow *)

function TGrayForm.FindAndRemoveGrayItem( aForm : TForm ) : boolean;
(* poistaa grayitems taulukosta ja palauttaa hintit jos tarvis *)
begin
  result := FindGrayWindow( aForm );
  if result then
  with GrayItem^ do
  begin
    (* palautetaan hinttien ohjaus *)
    if GrayWindow.GrayOwner = nil then
      Application.OnHint := FOldHint;

    (* poistetaan taulukosta *)
    GrayItems.Remove( GrayItem );
  end;(* with *)
end;(* RemoveGrayItem *)

procedure TGrayForm.DoGrayDestroy(Sender : TObject);
begin
  (* nollataan kahva *)
  if FindAndRemoveGrayItem( TForm( sender )) then
    dispose( GrayItem );
end;(* DoGrayDestroy *)

constructor TGrayForm.Create(AOwner: TComponent);
begin
  inherited Create( aOwner );

  GrayItems := TList.create;
  FEnabled := true;
end;(* Create *)

destructor TGrayForm.Destroy;
begin
  FreeAllItems;
  GrayItems.Free;
  inherited Destroy;
end;(* destroy *)

procedure TGrayForm.OpenGray( aForm : TForm );
(* 1. aForm = nil -> koko ruudun kokoinen
   2. aForm <> nil -> formin kokoinen *)
var
  DesktopHWnd: HWnd;
  DesktopDC: HDc;
begin
  (* jos ei ole kytss *)
  if not FEnabled then exit;

  new( GrayItem );
  With GrayItem^ do
  begin
    (* Luodaan Black *)
    GrayWindow := TGrayWindow.Create( application );
    with GrayWindow do
    begin
      borderstyle := bsnone;
      OnDestroy := DoGrayDestroy;
      GrayOwner := aForm;

      (* peiton koko *)
      if aForm = nil then
      begin
        (* koko ruudun peittv *)
        SetWindowPos( Handle, HWnd_Top, 0,0,screen.width, screen.height, SWP_NOREDRAW );
        (* hint tekstit ohjataan siihen *)
        Panel_Hint.visible := true;
        (* panelin tausta vri hint vriksi *)
        Panel_Hint.Color := Application.HintColor;
        (* aktivoidaan hinttien ohjaus *)
        FOldHint := Application.OnHint;
        Application.OnHint := DisplayHint;
      end else
        SetWindowPos( Handle, HWnd_Top, aform.left,aform.top,aform.width,aform.height, SWP_NOREDRAW);

      (* kuvan koko *)
      if aForm = nil then
        image.setbounds( 0,0, Screen.width, screen.height )
      else
        image.setbounds( 0, 0, aform.width, aform.height );

      (* listn harmaa huntu imageen *)
      image.canvas.pen.mode := pmxor;
      image.canvas.brush.color := 8421631;
      image.canvas.brush.color := 4210688;
      image.canvas.pen.color := 0;
      image.canvas.rectangle( 0,0,image.width, image.height );

      (* kahva nkyvn ty pytn *)
      DesktopHWnd := GetDesktopWindow;
      DesktopDC   := GetDC(DesktopHWnd);
      try
        (* kopoidaan bittikartta imageen *)
        if aForm = nil then
          BitBlt( image.canvas.Handle, 1, 1, image.width, image.height,
                  DesktopDC, 0,0, MERGECOPY )
        else
          BitBlt( image.canvas.Handle, 1, 1, image.width, image.height,
                  DesktopDC, aform.left, aform.top,MERGECOPY);
      finally
        (* vapautetaan typyt kahva *)
        ReleaseDC(DesktopHWnd, DesktopDC);
      end;(* try *)

      (* peitto nkyviin *)
      visible := true;
    end;(* with GrayWindow *)
  end;(* with GrayItem *)

  (* listn uusi item *)
  GrayItems.Add( GrayItem );
end;(* OpenGray *)

procedure TGrayForm.CloseGray( aForm : TForm );
(* suljetaan peitto
   1. etsitn oikea
   2. sulj... *)
begin
  (* jos ei ole kytss *)
  if not FEnabled then exit;

  if FindAndRemoveGrayItem( aForm ) then
  with GrayItem^ do
  begin
    (* suljetaan peitto *)
    GrayWindow.Visible := false;
    GrayWindow.Release;

    dispose( GrayItem );
  end;(* with GrayItem^ do *)
end;(* CloseGray *)

begin
  GrayShade := TGrayForm.Create( application );
end.
